///////////////////////////////////////////////////////////////////  
//						USB tester
//						
//TESTING ONE, TWO, THREE.
///////////////////////////////////////////////////////////////////  

#include "base.h"
#include "fstream.h"
#include "iostream.h"
//#include "defs.h"
#include "stdio.h"
#include "math.h"
#include "algo.h"  //windows
//#include <algorithm.h> //linux

unsigned short buffer[BUFFERSIZE];//readout buffer

IMPLEMENT_APP(TestApp)
///////////////////////////////////////////////////////////////////  
//ID's or identifiers for events where 'about' is set to be the #1 for compatibility reasons.
enum{
ID_About=1,
ID_Quit,
ID_Commandline
};
///////////////////////////////////////////////////////////////////  
//Another magic macro for our convenience.
//This table links system events to specific methods. AFAIK, that is.
BEGIN_EVENT_TABLE(MainWND,wxFrame)
	EVT_MENU(ID_Quit,MainWND::OnQuit)
	EVT_MENU(ID_About,MainWND::OnAbout)
	EVT_TEXT_ENTER(ID_Commandline, MainWND::OnEnter)
END_EVENT_TABLE()
///////////////////////////////////////////////////////////////////  
//This function is called when the program is started
bool TestApp::OnInit()
{
	MainWND *frame = new MainWND("USB tester", wxPoint(300,160), wxSize(400,400));

	frame->Show(true);
	SetTopWindow(frame);
	
	return true;
}
///////////////////////////////////////////////////////////////////  
//Main window constructor. Sets the apearance of main window (frame) and
//creates everything that is needed to make the window appear as if it was a competent
//windows application.
MainWND::MainWND(const wxString title, const wxPoint& pos, const wxSize& size):
wxFrame((wxFrame*)NULL,-1,title,pos,size,wxDEFAULT_FRAME_STYLE)
{					
	TextFrame = new wxTextCtrl(this,-1,"",		//This your standard read-only textfield.
	wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE|wxTE_READONLY);
    Commandline = new wxTextCtrl(this,ID_Commandline,"",wxDefaultPosition,wxDefaultSize,wxTE_PROCESS_ENTER);
    BaseSizer = new wxBoxSizer(wxVERTICAL);
    BaseSizer->Add(TextFrame,1,wxEXPAND,0);
    BaseSizer->Add(Commandline,0,wxEXPAND,0);
    this->SetSizer(BaseSizer);
    Commandline->SetFocus();
	// This section of the code generates the menus and the menubar of the app window.
	wxMenuBar *menubar = new wxMenuBar;		//Menubar is connected to handle smaller menus
											//that are to be declared on the following lines.
	wxMenu *file = new wxMenu;				//This is the 'File' menu.
	wxMenu *about = new wxMenu;				//...'About' menu
	//Following 5 lines add some content to those menus and link them to events.
	//Notice the use of ID's. If they are matched right there should be no problems.
	about->Append(ID_About,"&About","About this program");
	file->Append(ID_Quit,"&Quit","Quit this application");
	//Add the smaller menus to the menubar.
	menubar->Append(file,"&File");
	menubar->Append(about,"&About");
	SetMenuBar(menubar);	//Finally show the result.
	// This is where the menu section pretty much ends.
	CreateStatusBar();									//Create...yes.
    GoodData = false;
    lastGoodData = new unsigned short[BUFFERSIZE];
  
    //LOAD_PED();
    //LOAD_DNL_LUT();
}
///////////////////////////////////////////////////////////////////  
void MainWND::OnQuit(wxCommandEvent& WXUNUSED(event))
{
	Close(true);				//Close the program...goes to the destructor of MainWND where
								//there are some surprises for curious people.
}
///////////////////////////////////////////////////////////////////  
void MainWND::OnAbout(wxCommandEvent& WXUNUSED(event))
{
	wxMessageBox("USB tester application.\n - The year was 2009 - ","USB tester");
}
///////////////////////////////////////////////////////////////////  
void MainWND::OnEnter(wxCommandEvent &event)
{   
    
    if(Commandline->GetValue()== wxT("r"))
    {
       Clear_terminal();
       wxYield();
       SOFT_TRIG();
       wxYield();
       Read(true);
       wxYield();
    }
    else if(Commandline->GetValue() == wxT("loadped"))
    {
        Clear_terminal(); 
        LOAD_PED();
    }
    else if(Commandline->GetValue() == wxT("test"))
    {
        Clear_terminal(); 
        //TEST_FUNCTION();
    }
    else if(Commandline->GetValue() == wxT("read"))
    {
      Clear_terminal();
      LOG_DATA();

    }
    else if(Commandline->GetValue() == wxT("read_dc"))
    {
      Clear_terminal();
      LOG_DATA_DC();

    }
    else if(Commandline->GetValue() == wxT("ped"))
    {
        Clear_terminal();
        //PED_SCAN(500);
        //wxMilliSleep(3000);
        GENERATE_PED();
    }
    else if(Commandline->GetValue() == wxT("dnl"))
    {
        Clear_terminal();
        LOAD_DNL_LUT();
    }
    else if(Commandline->GetValue() == wxT("scan"))
    {
        Clear();
        DAC_SCAN();
    }
    else if(Commandline->GetValue() == wxT("level"))
    {
        Clear();
        int level = 2300;
        PED_SCAN(level);
        float voltage = level*1200/4096;
        TextFrame->AppendText(wxString::Format(wxT("ped voltage = %6.0f mV"),voltage));
    }
    else if(Commandline->GetValue() == wxT("sync"))
    {
        Clear();
        USB_SYNC(true);
        TextFrame->AppendText(wxString::Format(wxT("USB SYNC MODE")));
    }
    else if(Commandline->GetValue() == wxT("unsync"))
    {
        Clear();
        USB_SYNC(false);
        TextFrame->AppendText(wxString::Format(wxT("USB SYNC MODE disabled")));
    }
    else if(Commandline->GetValue() == wxT("trig"))
    {
        Clear();
        TRIG_CNTRL();
    }
    else if(Commandline->GetValue() == wxT("init"))
    {
        Clear_terminal();
        TRIG_CNTRL();        
    }
    else
        Send();
}
///////////////////////////////////////////////////////////////////
void MainWND::LOG_DATA()
{
 
    LOAD_DNL_LUT();
    LOAD_PED();        
    unsigned int i,j,k;//,DEBUG;   
    unsigned int NUM_READS = 300;
    float sample;//[256];
    float DNL_temp;
    
    ofstream LOGGED_DATA; 
    LOGGED_DATA.open ("LOG_DATA.txt");
    ofstream LOGGED_DATA_CH3; 
    LOGGED_DATA_CH3.open ("LOG_DATA_CH3.txt");
    
    for(k=0;k<NUM_READS; k++){
        TextFrame->AppendText(wxString::Format(wxT("READOUT: %d of %d\n"),k+1,NUM_READS));
        SOFT_TRIG();//for debugging
        wxYield();
        Read(false);
        wxYield();        
        for(i=0;i<CHAN; i++){
            for(j=0;j<256; j++){
            
                sample = (float)RAW[i][j];
                DNL_temp = DNL_LUT[i][(int)sample];
              //  sample -= DNL_temp;        //diff non-linearity
                sample -= PED_DATA[i][j];  //usual ped substraction
                //sample *= factor[i];     //voltage-count conversion factor
                LOGGED_DATA<<sample<<"\n";     //uncomment for AC readout  
                if(i==2){
                    LOGGED_DATA_CH3 <<sample<<"\n";
                }         
            }
        }
        //median substraction (ONLY FOR DC readout!!!!)
     /*   sort(sample, sample+256);
        float med = (sample[128]+sample[129])/2;
        for(j=0;j<256; j++){
            temp = (float)RAW[j]; 
            temp -= PED_DATA[j];
            temp = temp*factor;
            temp -= med;
            LOGGED_DATA<<temp<<"\n";
        }    */
        
        wxMilliSleep(10);
        //}
   
    }    
    LOGGED_DATA.close();
    LOGGED_DATA_CH3.close();
    TextFrame->AppendText(wxString::Format(wxT("......DONE READ\n")));
}
/////////////////////////////////////////////////////////////////// 
void MainWND::LOG_DATA_DC()
{
   // LOAD_DNL_LUT();
    unsigned int i,j,k;//,DEBUG;   
    unsigned int NUM_READS = 100;
    float sample[CHAN][256];
    float med[CHAN];
    float temp;
    
    ofstream LOGGED_DATA_DC; 
    LOGGED_DATA_DC.open ("LOG_DATA_DC.txt");
    
    for(k=0;k<NUM_READS; k++){
        TextFrame->AppendText(wxString::Format(wxT("READOUT: %d of %d\n"),k+1,NUM_READS));
        SOFT_TRIG();//for debugging
        wxYield();
        Read(false);
        wxYield();        
        for(i=0;i<CHAN; i++){
            med[i] = 0;
            temp = 0;
            for(j=0;j<256; j++){
            
                sample[i][j] = (float)RAW[i][j];
                //sample[j] = (float)RAW[j];
                /*for(int i=0;i<1000; i++){ 
                    if ((int)sample[j] == i && sample[j] > 200){
                    //  TextFrame->AppendText(wxString::Format(wxT("dnl %d  %f\n"),(int)sample[j],sample[j]));
                     sample[j] -= DNL_LUT[i];
                    }
                 }*/
                sample[i][j] -= PED_DATA[i][j];  //usual ped substraction
           // sample[j] = sample[j]*factor;
           // LOGGED_DATA<<sample<<"\n";     //uncomment for AC readout           
            }
        }
        //median substraction (ONLY FOR DC readout!!!!)
        for(i=0;i<CHAN; i++){
            sort(sample[i], sample[i]+256);
            med[i]=(sample[i][128]+sample[i][129])/2;
            for(j=0;j<256; j++){
                temp = (float)RAW[i][j]; 
                temp -= PED_DATA[i][j];
                //temp = temp*factor;
                temp -= med[i];
                LOGGED_DATA_DC<<temp<<"\n";
            }
           // LOGGED_DATA_DC<<temp<<"\n";
        }    //*/
        
       // wxMilliSleep(10);
        //}
   
    }    
    LOGGED_DATA_DC.close();
    TextFrame->AppendText(wxString::Format(wxT("......DONE READ\n")));
}
///////////////////////////////////////////////////////////////////////////// 
void MainWND::GENERATE_PED()
{
    unsigned int i,j,k;
    unsigned int num_reads = 300;
    unsigned int raw_array[CHAN][256][num_reads];   
    
    for(i=0;i<CHAN; i++){
        for(j=0;j<256; j++){
            for(k=0;k<num_reads;k++){
            raw_array[i][j][k]=0;
            }
        }
    }
       
    TextFrame->AppendText(wxString::Format(wxT("START PED_READOUT......\n")));
    
    for(k=0;k<num_reads; k++){
        TextFrame->AppendText(wxString::Format(wxT("PED_READOUT: %d of %d\n"),k+1,num_reads));
        SOFT_TRIG();
        wxYield();
        Read(false);
        
        wxYield();
        
        for(i=0;i<4; i++){
            for(j=0;j<256; j++){
                raw_array[i][j][k] = RAW[i][j];
            }
        } 
        //wxMilliSleep(10);
    }
    
    ofstream SAVE_PED;
    SAVE_PED.open ("PED_DATA.txt",ios::trunc);
    //need to do median ped subraction due to leakage issues
    int w = int(num_reads/2);
    for(i=0;i<CHAN; i++){
        for(j=0;j<256; j++){                              
            sort(raw_array[i][j], raw_array[i][j]+num_reads);
            PED_DATA[i][j] = raw_array[i][j][w];
            SAVE_PED<<PED_DATA[i][j]<<"\n";
        }
    }
    SAVE_PED.close();
    TextFrame->AppendText(wxString::Format(wxT("......DONE PED_READOUT\n")));
}
///////////////////////////////////////////////////////////////////
void MainWND::LOAD_PED()
{
    unsigned int i,j;
    ifstream LOAD_PED_DATA;
    LOAD_PED_DATA.open ("PED_DATA.txt");
  
    for(i=0;i<CHAN; i++){
        for(j=0;j<256; j++){
            LOAD_PED_DATA >> PED_DATA[i][j];
          //  TextFrame->AppendText(wxString::Format(wxT("cell %d %f\n"),j+1,PED_DATA[i][j]));
        }
    }    
    TextFrame->AppendText(wxString::Format(wxT("PED LOADED!\n")));
    LOAD_PED_DATA.close();
}
///////////////////////////////////////////////////////////////////
void MainWND::LOAD_DNL_LUT() //Loads DNL LUT + count/voltage conversion factors
{
    unsigned int i;
    //float sample;
    ifstream LOAD_ADC;
    ifstream LOAD_DNL;
    LOAD_DNL.open ("DC_LUT.txt");
    LOAD_ADC.open ("ADC_conversion.txt");
    for(i=0;i<CHAN; i++){
        LOAD_ADC >> factor[i];
        for(int j=0;j<2048; j++){     //11 bit LUT
            //LOAD_DNL >> smpl >> sample;
            //DNL_LUT[smpl] = sample;
            LOAD_DNL >> DNL_LUT[i][j];
            //TextFrame->AppendText(wxString::Format(wxT("DNL_LUT %f\n"),DNL_LUT[i][j]));
        }
    }
    TextFrame->AppendText(wxString::Format(wxT("DNL LOADED!\n")));
    LOAD_DNL.close();
    LOAD_ADC.close();
}
///////////////////////////////////////////////////////////////////  
void MainWND::Send()
{
    wxString temp;
	char *temp2;
	int i;
	unsigned long tempdata;
    unsigned int ManualData = 0;
    bool Send = false;
	temp = Commandline->GetValue().Lower();
	temp2 = (char*)temp.c_str();
	//Go through the character array to snoop about for characters other than digits and letters
	//from 'A' to 'F' (upper and lower case). We want to use hexadecimal values.
    for(i=0;i<(int)temp.Len();i++)
	{   
		if( ( (temp2[i] < '0') || (temp2[i] > '9') ) &&
		    ( (temp2[i] < 'a') || (temp2[i] > 'f') )
           )
		{
			wxMessageBox(wxT("Invalid input"),wxT("Erreur!"));
			Commandline->Clear();
			return;	
		}
	}
	//Empty textfield will yield a message stating that there's nothing to read.
	if(temp == wxT(""))
	{
		wxMessageBox(wxT("The field is empty."),wxT("Note"));
	}
	else	//Display the data that's to be sent.
	{
		temp.ToULong(&tempdata,16);		//Read the hex value from "temp" string buffer.
		ManualData = (unsigned int) tempdata;
        TextFrame->AppendText(wxString::Format(wxT("Sending 0x%x...\n"),ManualData));
        //Commandline->Clear();
        Send = true;
	}
    //Attempt to send raw data to the board.
    stdUSB out;
	if(Send == true)
	{
        if(out.createHandles() == stdUSB::SUCCEED)
        {
        	try
        	{
                out.sendData( ManualData );
                out.freeHandles();        
            }
            catch(...)
            {
                wxMessageBox(wxT("Could not send data."),wxT("USB Error"));    
                out.freeHandles();   
            }
        }
        else
            TextFrame->AppendText(wxT("Please connect the board.\n"));
    }
}
///////////////////////////////////////////////////////////////////  
void MainWND::Read(bool ENABLE_TXT)
{
    ofstream myfile2;
    stdUSB in;
    int i,j;
	unsigned short buffer[BUFFERSIZE];
	memset(buffer,0x0,BUFFERSIZE*sizeof(unsigned short));
	int samples;
	//DWORD dw;
	int dw;
	//http://www.codeguru.com/forum/archive/index.php/t-294690.html
	// says "DWORD is nothing more than a 32-bit integer type."
	GoodData = true;
	if(in.createHandles() == stdUSB::SUCCEED){
        try
    	{
            in.readData(buffer,BUFFERSIZE, &samples);
            tempStr = wxString::Format(wxT("Samples read %d\n"),samples);                                                   
            if(samples >= BUFFERSIZE){
                for(i=0;i<BUFFERSIZE;i++){
                    lastGoodData[i] = buffer[i];
                }
            }
            else
                GoodData = false;
            myfile2.open ("temp.txt",ios::trunc);
        for(i=2;i<1035;i++){
            //for(i=2;i<258;i++){
                myfile2<<buffer[i]<<" \n";
                //myfile2<<buffer[i]-DNL_LUT[i-2]<<" \n";
        		//tempStr += tempStr.Format(wxT("%x, "), buffer[i]);
        		tempStr += tempStr.Format(wxT("%x, "), buffer[i]);
            }
            myfile2.close();
           	tempStr += "\n";
            for(i=0;i<CHAN;i++){  
            //    for(j=0;j<249;j++){  
                for(j=0;j<256;j++){            	
           	        RAW[i][j] = buffer[2+256*i+j];
           	        //RAW[i][j] = buffer[j+2];
                }
            }
           //	ROVDD = lastGoodData[1025];            
            //PROVDD = lastGoodData[1026];           
           	if(ENABLE_TXT != false)
            {
                TextFrame->AppendText(tempStr);
            }
            in.freeHandles();
		}
		//catch (DWORD dw){
		catch (int dw){
			TextFrame->AppendText(wxString::Format(wxT("Errorcode %d\n"),dw));
		}
		catch(...){
			TextFrame->AppendText(wxT("Exception...probably a connection related issue.\n"));
		}
    }
    else
         TextFrame->AppendText(wxT("Please connect the board.\n"));
}
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
void MainWND::Clear()
{
    TextFrame->Clear();//clears terminal
    Commandline->Clear();//clears Commandline
}
///////////////////////////////////////////////////////////////////
void MainWND::Clear_terminal()
{
    TextFrame->Clear();//clears terminal
}
///////////////////////////////////////////////////////////////////
void MainWND::USB_SYNC(bool ENABLE)
{
    stdUSB in;
    in.createHandles();
    if(ENABLE != false)
    {
        in.sendData((unsigned int)0x00010001);//enable USB_SYNC
    }
    else
    {
        in.sendData((unsigned int)0x00000001);//disable USB_SYNC
    }
    in.freeHandles();
}
///////////////////////////////////////////////////////////////////
void MainWND::SOFT_TRIG()
{
    stdUSB in;
    in.createHandles();
    in.sendData((unsigned int)0x00000002);//software trigger
    in.freeHandles();
}
///////////////////////////////////////////////////////////////////
void MainWND::PED_SCAN(unsigned int ADC_VALUE)
{
    stdUSB in;
    in.createHandles();
    const unsigned int mask = 0x00000003;
    unsigned int ManualData;
    ManualData = ADC_VALUE << 16 | mask;//DTRIG discrimator threshold
    in.sendData(ManualData);
    in.freeHandles();
}
///////////////////////////////////////////////////////////////////
void MainWND::USB_DEBUG(unsigned int DEBUG)
{
    stdUSB in;
    in.createHandles();
    const unsigned int mask = 0x000000FF;
    unsigned int ManualData;
    ManualData = DEBUG << 16 | mask;
    in.sendData(ManualData);
    in.freeHandles();
}
///////////////////////////////////////////////////////////////////
void MainWND::TRIG_CNTRL()
{
    bool MODE = 0;  //trig controls
    bool SIGN = 0;
    bool SSlsb = 0;   // sampling speed controls
    bool SSmsb = 0;
    unsigned int THRESH = 0xFFF;
    
    if (MODE != 1){
        USB_SYNC(false);
        TextFrame->AppendText(wxString::Format(wxT("USB SYNC MODE disabled \n")));
    }
    else{    
        USB_SYNC(true);
        TextFrame->AppendText(wxString::Format(wxT("USB SYNC MODE enabled \n")));
    }
    
    TRIG_SET = (SSmsb << 15) | (SSlsb << 14) |(MODE << 13) | (SIGN << 12) | THRESH; //16 bit control word
    TextFrame->AppendText(wxString::Format(wxT("sampling/trig settings = 0x%x \n"), TRIG_SET));
    USB_DEBUG(TRIG_SET);
    
} 
///////////////////////////////////////////////////////////////////
void MainWND::DAC_SCAN()
{
    char stitle8[100];
    ofstream myfile8;
    sprintf(stitle8, "DAC_SCAN.txt");
    myfile8.open (stitle8, ios::trunc);
  /*  char stitle9[100];
    ofstream myfile9;
    sprintf(stitle9, "DAC_SCAN_cell.txt");
    myfile9.open (stitle9, ios::trunc);
    
    int cell = 25;*/
//i = 2000;
    float temp[CHAN];
    
    for(int i=10; i<3410; i+=20){        //dac control
        for(int k=0;k<CHAN;k++){
            temp[k] = 0;
        }
     //   float temp_cell =0;
        PED_SCAN(i);
        TextFrame->AppendText(wxString::Format(wxT("waiting .....PED =%d \n"), i));
        wxMilliSleep(2000);
        GENERATE_PED();
        
        for(int m=0;m<CHAN; m++){
            for(int j=0;j<256; j++){
                temp[m] += (float)PED_DATA[m][j];
            }
           /* if(j==cell){
                temp_cell = PED_DATA[j];
            }    */     
            temp[m] /= 256;
           //output to single column
           // myfile8 << temp[m] << "\n";
            
        }
        
        //or...output to .csv file
        myfile8 << temp[0] << ","<< temp[1] << ","<< temp[2] << ","<< temp[3] << "\n";
      //  myfile9 << temp_cell << "\n";
    }
    
    myfile8.close();
   // myfile9.close();
        
}         

